@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Type::Coercion::Union::AUTHORITY = 'cpan:TOBYINK';
-	$Type::Coercion::Union::VERSION   = '0.044';
+	$Type::Coercion::Union::VERSION   = '1.004004';
 }
 
 use Scalar::Util qw< blessed >;
@@ -31,7 +31,7 @@ sub _maybe_restore_type_constraint
 	{
 		return Type::Tiny::Union->new(type_constraints => $union);
 	}
-	return;
+	return; # uncoverable statement
 }
 
 sub type_coercion_map
@@ -57,7 +57,33 @@ sub add_type_coercions
 	_croak "Adding coercions to Type::Coercion::Union not currently supported" if @_;
 }
 
-# sub _build_moose_coercion ???
+sub _build_moose_coercion
+{
+	my $self = shift;
+	
+	my %options = ();
+	$options{type_constraint} = $self->type_constraint if $self->has_type_constraint;
+	
+	require Moose::Meta::TypeCoercion::Union;
+	my $r = "Moose::Meta::TypeCoercion::Union"->new(%options);
+	
+	return $r;
+}
+
+sub can_be_inlined
+{
+	my $self = shift;
+	
+	Types::TypeTiny::TypeTiny->assert_valid(my $type = $self->type_constraint);
+	
+	for my $tc (@$type)
+	{
+		next unless $tc->has_coercion;
+		return !!0 unless $tc->coercion->can_be_inlined;
+	}
+	
+	!!1;
+}
 
 1;
 
@@ -100,7 +126,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
