@@ -10,11 +10,10 @@ BEGIN {
 
 BEGIN {
 	$Type::Tiny::AUTHORITY   = 'cpan:TOBYINK';
-	$Type::Tiny::VERSION     = '1.000005';
-	$Type::Tiny::XS_VERSION  = '0.010';
+	$Type::Tiny::VERSION     = '1.004004';
+	$Type::Tiny::XS_VERSION  = '0.011';
 }
 
-use Eval::TypeTiny ();
 use Scalar::Util qw( blessed weaken refaddr isweak );
 use Types::TypeTiny ();
 
@@ -59,24 +58,55 @@ use overload
 	q(&{})     => "_overload_coderef",
 	q(|)       => sub {
 		my @tc = _swap @_;
-		if (!_FIXED_PRECEDENCE && !blessed $tc[0] && ref $tc[0] eq 'ARRAY') {
-			require Type::Tiny::_HalfOp;
-			return "Type::Tiny::_HalfOp"->new('|', @tc);
-		}
+		if (!_FIXED_PRECEDENCE && $_[2]) {
+			if (blessed $tc[0]) {
+				if (blessed $tc[0] eq "Type::Tiny::_HalfOp") {
+					my $type  = $tc[0]->{type};
+					my $param = $tc[0]->{param};
+					my $op    = $tc[0]->{op};
+					require Type::Tiny::Union;
+					return "Type::Tiny::_HalfOp"->new(
+						$op,
+						$param,
+						"Type::Tiny::Union"->new(type_constraints => [$type, $tc[1]]),
+					);
+				}
+			}
+			elsif (ref $tc[0] eq 'ARRAY') {
+				require Type::Tiny::_HalfOp;
+				return "Type::Tiny::_HalfOp"->new('|', @tc);
+			}
+	}
 		require Type::Tiny::Union;
-		"Type::Tiny::Union"->new(type_constraints => \@tc)
+		return "Type::Tiny::Union"->new(type_constraints => \@tc)
 	},
 	q(&)       => sub {
 		my @tc = _swap @_;
-		if (!_FIXED_PRECEDENCE && !blessed $tc[0] && ref $tc[0] eq 'ARRAY') {
-			require Type::Tiny::_HalfOp;
-			return "Type::Tiny::_HalfOp"->new('&', @tc);
-		}
+		if (!_FIXED_PRECEDENCE && $_[2]) {
+			if (blessed $tc[0]) {
+				if (blessed $tc[0] eq "Type::Tiny::_HalfOp") {
+					my $type  = $tc[0]->{type};
+					my $param = $tc[0]->{param};
+					my $op    = $tc[0]->{op};
+					require Type::Tiny::Intersection;
+					return "Type::Tiny::_HalfOp"->new(
+						$op,
+						$param,
+						"Type::Tiny::Intersection"->new(type_constraints => [$type, $tc[1]]),
+					);
+				}
+			}
+			elsif (ref $tc[0] eq 'ARRAY') {
+				require Type::Tiny::_HalfOp;
+				return "Type::Tiny::_HalfOp"->new('&', @tc);
+			}
+	}
 		require Type::Tiny::Intersection;
 		"Type::Tiny::Intersection"->new(type_constraints => \@tc)
 	},
 	q(~)       => sub { shift->complementary_type },
 	q(==)      => sub { $_[0]->equals($_[1]) },
+	q(!=)      => sub { not $_[0]->equals($_[1]) },
 	q(<)       => sub { my $m = $_[0]->can('is_subtype_of'); $m->(_swap @_) },
 	q(>)       => sub { my $m = $_[0]->can('is_subtype_of'); $m->(reverse _swap @_) },
 	q(<=)      => sub { my $m = $_[0]->can('is_a_type_of');  $m->(_swap @_) },
@@ -119,11 +149,30 @@ our %ALL_TYPES;
 
 my $QFS;
 my $uniq = 1;
+my $subname;
 sub new
 {
 	my $class  = shift;
 	my %params = (@_==1) ? %{$_[0]} : @_;
 	
+	if (exists $params{constraint}
+	and not ref $params{constraint}
+	and not exists $params{constraint_generator}
+	and not exists $params{inline_generator})
+	{
+		require Eval::TypeTiny;
+		my $code = $params{constraint};
+		$params{constraint} = Eval::TypeTiny::eval_closure(
+			source      => sprintf('sub ($) { %s }', $code),
+			description => "anonymous check",
+		);
+		$params{inlined} ||= sub {
+			my ($type) = @_;
+			my $inlined = $_ eq '$_' ? "do { $code }" : "do { local \$_ = $_; $code }";
+			$type->has_parent ? (undef, $inlined) : $inlined;
+		};
+	}
+	
 	if (exists $params{parent})
 	{
 		$params{parent} = ref($params{parent}) =~ /^Type::Tiny\b/
@@ -132,8 +181,16 @@ sub new
 		
 		_croak "Parent must be an instance of %s", __PACKAGE__
 			unless blessed($params{parent}) && $params{parent}->isa(__PACKAGE__);
+		
+		if ($params{parent}->deprecated and not exists $params{deprecated})
+		{
+			$params{deprecated} = 1;
+		}
 	}
 	
+	# canonicalize to a boolean
+	$params{deprecated} = !!$params{deprecated};
+	
 	$params{name} = "__ANON__" unless exists $params{name};
 	$params{uniq} = $uniq++;
 	
@@ -203,14 +260,19 @@ sub new
 		$self->coercion->add_type_coercions(@$arr);
 	}
 	
-	if ($params{my_methods} and eval { require Sub::Name })
+	if ($params{my_methods})
 	{
-		for my $key (keys %{$params{my_methods}})
+		$subname =
+			eval { require Sub::Util } ? \&Sub::Util::set_subname :
+			eval { require Sub::Name } ? \&Sub::Name::subname :
+			0
+			if not defined $subname;
+		if ($subname)
 		{
-			Sub::Name::subname(
-				sprintf("%s::my_%s", $self->qualified_name, $key),
-				$params{my_methods}{$key},
-			);
+			(Scalar::Util::reftype($params{my_methods}{$_}) eq 'CODE') && $subname->(
+				sprintf("%s::my_%s", $self->qualified_name, $_),
+				$params{my_methods}{$_},
+			) for keys %{$params{my_methods}};
 		}
 	}
 	
@@ -280,6 +342,7 @@ sub coercion                 { $_[0]{coercion}       ||= $_[0]->_build_coercion
 sub message                  { $_[0]{message} }
 sub library                  { $_[0]{library} }
 sub inlined                  { $_[0]{inlined} }
+sub deprecated               { $_[0]{deprecated} }
 sub constraint_generator     { $_[0]{constraint_generator} }
 sub inline_generator         { $_[0]{inline_generator} }
 sub name_generator           { $_[0]{name_generator} ||= $_[0]->_build_name_generator }
@@ -363,6 +426,7 @@ sub _build_compiled_check
 		return $self->parent->compiled_check;
 	}
 	
+	require Eval::TypeTiny;
 	return Eval::TypeTiny::eval_closure(
 		source      => sprintf('sub ($) { %s }', $self->inline_check('$_[0]')),
 		description => sprintf("compiled check '%s'", $self),
@@ -673,7 +737,7 @@ sub inline_check
 			unless $self->has_parent;
 		$r[0] = $self->parent->inline_check(@_);
 	}
-	my $r = join " && " => map { /[;{}]/ ? "do { $_ }" : "($_)" } @r;
+	my $r = join " && " => map { /[;{}]/ && !/\Ado \{.+\}\z/ ? "do { $_ }" : "($_)" } @r;
 	return @r==1 ? $r : "($r)";
 }
 
@@ -1231,9 +1295,12 @@ in reading when dealing with type constraint objects.
 
 =item C<< constraint >>
 
-Coderef to validate a value (C<< $_ >>) against the type constraint. The
-coderef will not be called unless the value is known to pass any parent
-type constraint (see C<parent> below).
+Coderef to validate a value (C<< $_ >>) against the type constraint.
+The coderef will not be called unless the value is known to pass any
+parent type constraint (see C<parent> below).
+
+Alternatively, a string of Perl code checking C<< $_ >> can be passed
+as a parameter to the constructor, and will be converted to a coderef.
 
 Defaults to C<< sub { 1 } >> - i.e. a coderef that passes all values.
 
@@ -1251,6 +1318,7 @@ type. Optional.
 
 If C<constraint> (above) is a coderef generated via L<Sub::Quote>, then
 Type::Tiny I<may> be able to automatically generate C<inlined> for you.
+If C<constraint> (above) is a string, it will be able to.
 
 =item C<< name >>
 
@@ -1272,6 +1340,15 @@ The package name of the type library this type is associated with.
 Optional. Informational only: setting this attribute does not install
 the type into the package.
 
+=item C<< deprecated >>
+
+Optional boolean indicating whether a type constraint is deprecated.
+L<Type::Library> will issue a warning if you attempt to import a deprecated
+type constraint, but otherwise the type will continue to function as normal.
+There will not be deprecation warnings every time you validate a value, for
+instance. If omitted, defaults to the parent's deprecation status (or false
+if there's no parent).
+
 =item C<< message >>
 
 Coderef that returns an error message when C<< $_ >> does not validate
@@ -1305,26 +1382,57 @@ constraints. Unlike Moose, these aren't handled by separate subclasses.
 
 =item C<< constraint_generator >>
 
-Coderef that generates a new constraint coderef based on parameters.
-Alternatively, the constraint generator can return a fully-formed
-Type::Tiny object, in which case the C<name_generator>, C<inline_generator>,
-and C<coercion_generator> attributes documented below are ignored.
+Coderef that is called when a type constraint is parameterized. When called,
+it is passed the list of parameters, though any parameter which looks like a
+foreign type constraint (Moose type constraints, Mouse type constraints, etc,
+I<< and coderefs(!!!) >>) is first coerced to a native Type::Tiny object.
+
+Note that for compatibility with the Moose API, the base type is I<not>
+passed to the constraint generator, but can be found in the package variable
+C<< $Type::Tiny::parameterize_type >>. The first parameter is also available
+as C<< $_ >>.
+
+Types I<can> be parameterized with an empty parameter list. For example,
+in L<Types::Standard>, C<Tuple> is just an alias for C<ArrayRef> but
+C<< Tuple[] >> will only allow zero-length arrayrefs to pass the constraint.
+If you wish C<< YourType >> and C<< YourType[] >> to mean the same thing,
+then do:
+
+ return $Type::Tiny::parameterize_type unless @_;
+
+The constraint generator should generate and return a new constraint coderef
+based on the parameters. Alternatively, the constraint generator can return a
+fully-formed Type::Tiny object, in which case the C<name_generator>,
+C<inline_generator>, and C<coercion_generator> attributes documented below
+are ignored.
 
 Optional; providing a generator makes this type into a parameterizable
-type constraint.
+type constraint. If there is no generator, attempting to parameterize the
+type constraint will throw an exception.
 
 =item C<< name_generator >>
 
-A coderef which generates a new display_name based on parameters.
+A coderef which generates a new display_name based on parameters. Called with
+the same parameters and package variables as the C<constraint_generator>.
+Expected to return a string.
+
 Optional; the default is reasonable.
 
 =item C<< inline_generator >>
 
-A coderef which generates a new inlining coderef based on parameters.
+A coderef which generates a new inlining coderef based on parameters. Called
+with the same parameters and package variables as the C<constraint_generator>.
+Expected to return a coderef.
+
+Optional.
 
 =item C<< coercion_generator >>
 
 A coderef which generates a new L<Type::Coercion> object based on parameters.
+Called with the same parameters and package variables as the
+C<constraint_generator>. Expected to return a blessed object.
+
+Optional.
 
 =item C<< deep_explanation >>
 
@@ -1478,6 +1586,11 @@ get a similar result using overloaded C<< & >>:
 
    HashRef & sub { exists($_->{name}) }
 
+Like the C<< constraint >> attribute, this will accept a string of Perl
+code:
+
+   HashRef->where('exists($_->{name})')
+
 =item C<< child_type_class >>
 
 The class that create_child_type will construct by default.
@@ -1756,6 +1869,18 @@ for easy localization.
 
 =back
 
+=head2 Environment
+
+=over
+
+=item C<PERL_TYPE_TINY_XS>
+
+Currently this has more effect on L<Types::Standard> than Type::Tiny. In
+future it may be used to trigger or suppress the loading XS implementations
+of parts of Type::Tiny.
+
+=back
+
 =head1 BUGS
 
 Please report any bugs to
@@ -1790,7 +1915,7 @@ Thanks to Matt S Trout for advice on L<Moo> integration.
 
 =head1 COPYRIGHT AND LICENCE
 
-This software is copyright (c) 2013-2014 by Toby Inkster.
+This software is copyright (c) 2013-2014, 2017-2019 by Toby Inkster.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
